#!/usr/bin/perl -w
#
# file    : fci-client.pl
# changed : 2005-02-08
# author  : Bas Couwenberg <sebastic@xs4all.nl>
#

use strict;
use LWP::UserAgent;
use HTTP::Request::Common;
use File::Basename;
use Getopt::Mixed "nextOption";

# default configuration
my $qd_bin  = dirname($0).'/qd';
my $version = '0.3';

chomp(my $hostname = `hostname`);

my %cfg = (
		self		=> 'fci-client.pl',
		version		=> $version,
		url 		=> 'http://example.com/index.pl',
		client_name	=> $hostname,
		folding_dir	=> '/home/folding/',
		useragent	=> 'fci-client/'.$version,
		verbose		=> 0,
		update_qd	=> 0,
		qd_url		=> "http://www.boston.quik.com/rph/",
		qdinfo_file	=> dirname($0).'/qdinfo.dat'
	);

$cfg{unitinfo_file} = $cfg{folding_dir}.'unitinfo.txt';
$cfg{config_file}   = $cfg{folding_dir}.'client.cfg';
$cfg{xyz_file}      = $cfg{folding_dir}.'work/current.xyz';

# get command line parameters
Getopt::Mixed::init("url=s client=s dir=s config=s unitinfo=s xyz=s useragent=s verbose help update-qd qd-url=s");
while (my ($option, $value) = nextOption()) {
	if($option eq 'help') {
		print_usage();
		Getopt::Mixed::cleanup();
		exit;
	}
	if($option eq 'dir') {
		$cfg{folding_dir} = $value;

		$cfg{unitinfo_file} = $cfg{folding_dir}.'unitinfo.txt';
		$cfg{config_file}   = $cfg{folding_dir}.'client.cfg';
		$cfg{xyz_file}      = $cfg{folding_dir}.'work/current.xyz';
	}
	
	$cfg{url} 	    = $value if($option eq 'url');
	$cfg{client_name}   = $value if($option eq 'client');
	$cfg{config_file}   = $value if($option eq 'config');
	$cfg{unitinfo_file} = $value if($option eq 'unitinfo');
	$cfg{xyz_file}      = $value if($option eq 'xyz');
	$cfg{useragent}	    = $value if($option eq 'useragent');
	$cfg{verbose} 	    = 1      if($option eq 'verbose');
	$cfg{update_qd}	    = 1      if($option eq 'update-qd');
	$cfg{qd_url}	    = $value if($option eq 'qd-url');
}
Getopt::Mixed::cleanup();

print basename($0)." started\n\n" if($cfg{verbose});

# check Folding@Home directory & useragent settings
if(!$cfg{useragent} || $cfg{useragent} eq '' || $cfg{useragent} =~ /^\s+$/) {
	print "Error: Invalid User-Agent! ($cfg{useragent})\n";
	exit;
}

my $ua  = LWP::UserAgent->new();
   $ua->agent($cfg{useragent});

if($cfg{update_qd}) {
	print "Fetching latest qd version from $cfg{qd_url}\n" if($cfg{verbose});

	my $qd_url     = $cfg{qd_url}.'qd';
	my $qdinfo_url = $cfg{qd_url}.'qdinfo.dat';

	if(!&invalid_url($qd_url) && !&invalid_url($qdinfo_url)) {
		print "\n" if($cfg{verbose});
		
		# download qd
		if(my $qd_data = fetch_url($qd_url)) {
			my $qd_file = dirname($0)."/qd";

			#save the file
			save_file($qd_file, $qd_data);
		}
		else {
			print "Error: Failed to retrieve URL! ($qd_url)\n";
		}

		print "\n" if($cfg{verbose});
		
		# download qdinfo.dat
		if(my $qdinfo_data = fetch_url($qdinfo_url)) {
			my $qdinfo_file = dirname($0)."/qdinfo.dat";

			#save the file
			save_file($qdinfo_file, $qdinfo_data);
		}
		else {
			print "Error: Failed to retrieve URL! ($qdinfo_url)\n";
		}
	}
	elsif(!&invalid_url($qd_url)) {
		print "Error: Invalid URL or HTTP error! ($qdinfo_url)\n";
	}
	else {
		print "Error: Invalid URL or HTTP error! ($qd_url)\n";
	}

	exit;
}

# check file existance & other parameters
if(!-r $cfg{folding_dir}) {
	print "Error: Cannot read Folding\@Home client directory! ($cfg{folding_dir})\n";
	exit;
}
elsif(!-r $cfg{config_file}) {
	print "Error: Cannot read client configuration file! ($cfg{config_file})\n";
	exit;
}
elsif(!-r $cfg{unitinfo_file}) {
	print "Error: Cannot read unitinfo file!\n";
	exit;
}
elsif(!-r $cfg{xyz_file}) {
	print "Error: Cannot read xyz file!\n";
	exit;
}
elsif(!$cfg{client_name} || $cfg{client_name} eq '' || $cfg{client_name} =~ /^\s+$/) {
	print "Error: Invalid client name! ($cfg{client_name})\n";
	exit;
}
elsif($cfg{url} !~ /^http:\/\// || &invalid_url($cfg{url})) {
	print "Error: Invalid URL or HTTP error! ($cfg{url})\n";
	exit;
}

# print configuration if verbose output is requested
print "Invocing `$qd_bin` to parse queue.dat file: \"$qd_bin\"-f \"$cfg{folding_dir}\" -n \"$cfg{qdinfo_file}\"\n\n" if($cfg{verbose});
my $qd_output = `"$qd_bin" -f "$cfg{folding_dir}" -n "$cfg{qdinfo_file}"`;

if($cfg{verbose}) {
	print "Going to submit the following data to $cfg{url}:\n";
	print "client name    : $cfg{client_name}\n";
	print "config file    : $cfg{config_file}\n";
	print "unitinfo file  : $cfg{unitinfo_file}\n";
	print "xyz file       : $cfg{xyz_file}";
	print "qd data        :\n<!-- start qd data -->\n$qd_output\n<!-- end qd data -->\n";
	print "\n";
}

# post file data to the server
my $req = POST $cfg{url},
	  Content_Type => 'form-data',
	  Content      => [
		  	   submit        => 1,
			   action        => 'upload',
			   client_name   => $cfg{client_name},
			   qd_output	 => $qd_output,
			   unitinfo_file => [ $cfg{unitinfo_file} ],
			   config_file   => [ $cfg{config_file} ],
			   xyz_file	 => [ $cfg{xyz_file} ]
			  ];

my $res = $ua->request($req);

if ($res->is_success()) {
	if($cfg{verbose} == 1) {
		print "Successfully posted data to the server.\n";
		print $res->content."\n";
	}
} 
else {
	print "Error uploading unitinfo file ($cfg{unitinfo_file}), config file ($cfg{config_file}) ";
	print "and queue.dat file ($cfg{queue_file}) ";
	print "to url ($cfg{url}) for client ($cfg{client_name}) with useragent ($cfg{useragent})!\n";
}

################[ Subroutines ]################

sub save_file {
	my $filename = $_[0];
	my $content  = $_[1];
	
	print "Saving to disc: $filename\n" if($cfg{verbose});
	if(!-e $filename || -w $filename) {
		open(F, "> $filename") || die "Cannot open file ($!)";
		print F $content;
		close F;
		
		return 1;
	}
	else {
		print "Error: Cannot write file! ($filename)\n";
		
		return undef;
	}
}

sub fetch_url {
	my $url = $_[0];
	
	my $ua  = LWP::UserAgent->new();
	   $ua->agent($cfg{useragent});
	
	my $req = HTTP::Request->new(GET => $url);
	
	print "Retrieving url: $url ... " if($cfg{verbose});
	my $res = $ua->request($req);
	if($res->is_success) {
		print "Success\n" if($cfg{verbose});
		
		return $res->content;
	}
	else {
		print "Failed!\n" if($cfg{verbose});
		print "Error: Failed to retrieve url! ($url)\n";
		
		return undef;
	}
}
																	
sub invalid_url {
	my $url = $_[0];

	my $ua  = LWP::UserAgent->new();
	   $ua->agent($cfg{useragent});
	
	my $req = HTTP::Request->new(HEAD => $url);
	
	my $res = $ua->request($req);
	if($res->is_success()) {
		return 0;
	}
	elsif($res->is_error()) {
		return 1;
	}
	else {
		return 1;
	}
}

sub print_usage {
	my $verbose    = $cfg{verbose}   ? 'on' : 'off';
	my $update_qd  = $cfg{update_qd} ? 'on' : 'off';
	chomp(my $qd_version = `"$qd_bin" -v`);
	
	print "$cfg{self} v$cfg{version}\n";
	print "with qd [$qd_version] by Dick Howell <rph\@boston.quik.com>\n";
	print "\n";
	print "Usage: ".basename($0)." [OPTIONS]\n\n";
	print "Options:\n";
	print "--url <url>        URL to POST the data to        (default: $cfg{url})\n";
	print "--client <name>    Client name                    (default: $cfg{client_name})\n";
	print "--dir <directory>  Folding\@Home client directory  (default: $cfg{folding_dir})\n";
	print "--config <file>    Path to the client.cfg file    (default: $cfg{config_file})\n";
	print "--unitinfo <file>  Path to the unitinfo.txt file  (default: $cfg{unitinfo_file})\n";
	print "--useragent <name> User-Agent string              (default: $cfg{useragent})\n";
	print "--verbose          Verbose (debug) output         (default: $verbose)\n";
	print "--help             Print this information\n";
	print "\n";
	print "Options for qd:\n";
	print "--update-qd        Download the latest qd version (default: $update_qd)\n";
	print "--qd-url <url>     URL to qd and qdinfo.dat       (default: $cfg{qd_url})\n";
}
